#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include "sareadline.h"
#include "strlist.h"
#include "tokenizer.h"

/*
	assign an opened file with the given flags and mode to fd
*/
void fassign ( int fd , char* path , int oflags , mode_t mode ) {
	int newfd = open( path , oflags , mode);
	if ( newfd < 0) {
		perror ( path ); exit (255);
	}
	if ( dup2(newfd , fd ) < 0) {
		perror ( "dup2" ); exit (255);
	}
	close (newfd );
}

int main () {
	stralloc line = {0};
	while ( printf ( "%% "), readline ( stdin , &line )) {
		strlist tokens = {0};
		stralloc_0 (&line ); /* required by tokenizer () */
		if (! tokenizer (&line , & tokens )) break;
		if ( tokens.len == 0) continue;
		pid_t child = fork ();
		if ( child == -1) {
			perror ( " fork" ); continue;
		}
		if ( child == 0) {
			strlist argv = {0}; /* list of arguments */
			char* cmdname = 0; /* first argument */
			char* path ; /* of output files */
			int oflags ;
	
			for ( int i = 0; i < tokens.len ; ++ i ) {
				switch ( tokens.list [ i ][0]) {
					case '<':
						fassign (0, & tokens.list [ i ][1], O_RDONLY, 0);
						break;
					case '>' :
						path = &tokens.list [ i ][1];
						oflags = O_WRONLY|O_CREAT;
						if (*path == '>' ) {
							++path ; oflags |= O_APPEND;
						} else {
							oflags |= O_TRUNC;
						}
						fassign (1, path , oflags , 0666);
						break;
					default :
						strlist_push (&argv , tokens .list[ i ]);
						if ( cmdname == 0) cmdname = tokens.list[ i ];
				}
			}
			if ( cmdname == 0) exit (0);
			strlist_push(&argv, 0);
			execvp (cmdname, argv.list );
			perror (cmdname);
			exit (255);
		}

	/* wait for termination of child */
	int stat ;
	pid_t pid = wait(&stat );
	if ( pid == child ) {
		if (WIFEXITED(stat)) {
			int code = WEXITSTATUS(stat);
			if ( code && code != 255) {
				printf ( "terminated with exit code %d\n", code);
			}
		} else {
			printf ( "terminated abnormally\n");
		}
	} else {
		perror ( "wait" );
	}
	}
}
